import numpy as np
import pickle
import pandas as pd
import optuna
from bvpTune.visualizer import Visualizer
from bvpTune.optimizer import Optimizer
import warnings
warnings.filterwarnings("ignore")

class bvpFineTuner:
    """class that handles the fine-tuning of congiguration parameters of bvp4c solver from MATLAB"""

    def __init__(self):
        self.optimizer = Optimizer()
        self.visualizer = Visualizer()

    def setTrailNumber(self, trials):
        self.optimizer.setTrailNumber(trials)

    def getSolvabiltyStatus(self, test_case_type, max_grid_points, newton_critical_tolerance,
                            newton_armijo_probes, newton_max_iterations, newton_tolerance,
                            add_factor, remove_factor, use_collocation_scaling):
        array = np.array([test_case_type, max_grid_points, np.log(newton_critical_tolerance),
                          newton_armijo_probes, newton_max_iterations, np.log(newton_tolerance),
                          add_factor, remove_factor, use_collocation_scaling])
        array = array.reshape(1, -1)

        if bool(self.optimizer.getSolvabiltyStatus(array)):
            return 'The problem is solvable'
        else:
            return 'The problem is not solvable'

    def getSolverPerformance(self, test_case_type, max_grid_points, newton_critical_tolerance,
                             newton_armijo_probes, newton_max_iterations, newton_tolerance,
                             add_factor, remove_factor, use_collocation_scaling):
        array = np.array([test_case_type, max_grid_points, np.log(newton_critical_tolerance),
                          newton_armijo_probes, newton_max_iterations, np.log(newton_tolerance),
                          add_factor, remove_factor, use_collocation_scaling])
        array = array.reshape(1, -1)

        if bool(self.optimizer.getSolvabiltyStatus(array)):
            stats = self.optimizer.getSolverPerformance(array)
            return print('\n', 'nODEevals: ', int(stats[0][0]), '\n', 'nGridPoints: ', int(stats[0][1]), '\n',
                         'maxResiduum: ',
                         stats[0][2], '\n')
        else:
            return 'The problem is not solvable'

    def getOptimalODEevals(self, test_case_type):
        print("Calculating the best setting")
        self.optimizer.setTestCaseType(test_case_type)
        best_trial = self.optimizer.singleCriterion('nODEevals')
        best_trial.params['newton_critical_tolerance'] = np.exp(best_trial.params['newton_critical_tolerance'])
        best_trial.params['newton_tolerance'] = np.exp(best_trial.params['newton_tolerance'])
        print('Numerical Settings: ', best_trial.params)
        print('ODE Evaluations: ', int(best_trial.values[0]))

    def getOptimalGridPoints(self, test_case_type):
        print("Calculating the best setting")
        self.optimizer.setTestCaseType(test_case_type)
        best_trial = self.optimizer.singleCriterion('nGridPoints')
        best_trial.params['newton_critical_tolerance'] = np.exp(best_trial.params['newton_critical_tolerance'])
        best_trial.params['newton_tolerance'] = np.exp(best_trial.params['newton_tolerance'])
        print('Numerical Settings: ', best_trial.params)
        print('Grid Points: ', int(best_trial.values[0]))

    def getOptimalResiduum(self, test_case_type):
        print("Calculating the best setting")
        self.optimizer.setTestCaseType(test_case_type)
        best_trial = self.optimizer.singleCriterion('maxResiduum')
        best_trial.params['newton_critical_tolerance'] = np.exp(best_trial.params['newton_critical_tolerance'])
        best_trial.params['newton_tolerance'] = np.exp(best_trial.params['newton_tolerance'])
        print('Numerical Settings: ', best_trial.params)
        print('Maximum Residdum: ', best_trial.values)

    def getOptimalODEevalsAndGridPoints(self, test_case_type):
        print("Calculating the pareto front settings")
        self.optimizer.setTestCaseType(test_case_type)
        best_trials = self.optimizer.twoCriteria('EvalGp')
        row = pd.DataFrame(
            columns=['test_case_type', 'max_grid_points', 'newton_critical_tolerance', 'newton_armijo_probes',
                     'newton_max_iterations', 'newton_tolerance', 'add_factor', 'remove_factor',
                     'use_collocation_scaling', 'ODE_Evaluations', 'Grid_Points'])
        for i in range(0, len(best_trials)):
            best_trials[i].params['newton_critical_tolerance'] = np.exp(
                best_trials[i].params['newton_critical_tolerance'])
            best_trials[i].params['newton_tolerance'] = np.exp(best_trials[i].params['newton_tolerance'])
            row.loc[i] = [test_case_type, best_trials[i].params['max_grid_points'],
                          best_trials[i].params['newton_critical_tolerance'],
                          best_trials[i].params['newton_armijo_probes'],
                          best_trials[i].params['newton_max_iterations'], best_trials[i].params['newton_tolerance'],
                          best_trials[i].params['add_factor'], best_trials[i].params['remove_factor'],
                          best_trials[i].params['use_collocation_scaling'], best_trials[i].values[0],
                          best_trials[i].values[1]]
        print("Completed")
        return row

    def getOptimalGridPointsAndResiduum(self, test_case_type):
        print("Calculating the pareto front settings")
        self.optimizer.setTestCaseType(test_case_type)
        best_trials = self.optimizer.twoCriteria('GpRes')
        row = pd.DataFrame(
            columns=['test_case_type', 'max_grid_points', 'newton_critical_tolerance', 'newton_armijo_probes',
                     'newton_max_iterations', 'newton_tolerance', 'add_factor', 'remove_factor',
                     'use_collocation_scaling', 'Grid_Points', 'Maximum_Residuum'])
        for i in range(0, len(best_trials)):
            best_trials[i].params['newton_critical_tolerance'] = np.exp(
                best_trials[i].params['newton_critical_tolerance'])
            best_trials[i].params['newton_tolerance'] = np.exp(best_trials[i].params['newton_tolerance'])
            row.loc[i] = [test_case_type, best_trials[i].params['max_grid_points'],
                          best_trials[i].params['newton_critical_tolerance'],
                          best_trials[i].params['newton_armijo_probes'],
                          best_trials[i].params['newton_max_iterations'], best_trials[i].params['newton_tolerance'],
                          best_trials[i].params['add_factor'], best_trials[i].params['remove_factor'],
                          best_trials[i].params['use_collocation_scaling'], best_trials[i].values[0],
                          best_trials[i].values[1]]
        print("Completed")
        return row

    def getOptimalResiduumAndODEevals(self, test_case_type):
        print("Calculating the pareto front settings")
        self.optimizer.setTestCaseType(test_case_type)
        best_trials = self.optimizer.twoCriteria('ResEval')
        row = pd.DataFrame(
            columns=['test_case_type', 'max_grid_points', 'newton_critical_tolerance', 'newton_armijo_probes',
                     'newton_max_iterations', 'newton_tolerance', 'add_factor', 'remove_factor',
                     'use_collocation_scaling', 'Maximum_Residuum', 'ODE_Evaluations'])
        for i in range(0, len(best_trials)):
            best_trials[i].params['newton_critical_tolerance'] = np.exp(
                best_trials[i].params['newton_critical_tolerance'])
            best_trials[i].params['newton_tolerance'] = np.exp(best_trials[i].params['newton_tolerance'])
            row.loc[i] = [test_case_type, best_trials[i].params['max_grid_points'],
                          best_trials[i].params['newton_critical_tolerance'],
                          best_trials[i].params['newton_armijo_probes'],
                          best_trials[i].params['newton_max_iterations'], best_trials[i].params['newton_tolerance'],
                          best_trials[i].params['add_factor'], best_trials[i].params['remove_factor'],
                          best_trials[i].params['use_collocation_scaling'], best_trials[i].values[0],
                          best_trials[i].values[1]]
        print("Completed")
        return row

    def getOptimizedSettings(self, test_case_type):
        print("Calculating the pareto front settings")
        self.optimizer.setTestCaseType(test_case_type)
        best_trials = self.optimizer.threeCriteria()
        row = pd.DataFrame(
            columns=['test_case_type', 'max_grid_points', 'newton_critical_tolerance', 'newton_armijo_probes',
                     'newton_max_iterations', 'newton_tolerance', 'add_factor', 'remove_factor',
                     'use_collocation_scaling', 'ODE_Evaluations', 'Grid_Points', 'Maximum_Residuum'])
        for i in range(0, len(best_trials)):
            best_trials[i].params['newton_critical_tolerance'] = np.exp(
                best_trials[i].params['newton_critical_tolerance'])
            best_trials[i].params['newton_tolerance'] = np.exp(best_trials[i].params['newton_tolerance'])

            row.loc[i] = [test_case_type, best_trials[i].params['max_grid_points'],
                          best_trials[i].params['newton_critical_tolerance'],
                          best_trials[i].params['newton_armijo_probes'],
                          best_trials[i].params['newton_max_iterations'], best_trials[i].params['newton_tolerance'],
                          best_trials[i].params['add_factor'], best_trials[i].params['remove_factor'],
                          best_trials[i].params['use_collocation_scaling'], best_trials[i].values[0],
                          best_trials[i].values[1],
                          best_trials[i].values[2]]
        print("Completed")
        return row

    def visualize(self, dataframe):
        if len(dataframe.columns) == 11:
            self.visualizer.visualize2d(dataframe)
        elif len(dataframe.columns) == 12:
            self.visualizer.visualize3d(dataframe)
        else:
            print('Dataframe is not in right format. Please provided the dataframe returned by bvpFineTuner functions')



#!/usr/bin/env python
# coding: utf-8

# In[1]:


import pandas as pd
import numpy as np
from sklearn.model_selection import train_test_split
from sklearn.preprocessing import StandardScaler
import lightgbm as lgb
from sklearn.ensemble import RandomForestClassifier
from sklearn.neighbors import KNeighborsClassifier
import sklearn
from keras.models import Sequential
from keras.layers import Dense
from keras import optimizers
import keras
from sklearn.metrics import precision_score, recall_score, accuracy_score
from sklearn.metrics import confusion_matrix
import pickle


# In[3]:


def train_test_split(data):
    Input = pd.DataFrame()
    Output = pd.DataFrame()

    Input['max_grid_points'] = data['max_grid_points']
    Input['newton_critical_tolerance'] = np.log(data['newton_critical_tolerance'])
    Input['newton_armijo_probes'] = data['newton_armijo_probes']
    Input['newton_max_iterations'] = data['newton_max_iterations']
    Input['newton_tolerance'] = np.log(data['newton_tolerance'])
    Input['add_factor'] = data['add_factor']
    Input['remove_factor'] = data['remove_factor']
    Input['use_collocation_scaling'] = data['use_collocation_scaling']

    Output['success'] = data['success']

    X_train, X_test, y_train, y_test = sklearn.model_selection.train_test_split(Input, Output, train_size = 0.8, 
                                                                                test_size = 0.2, random_state=101)
    
    #normalize the input features
    scaler = StandardScaler()
    X_train_scaled = scaler.fit_transform(X_train)
    X_test_scaled = scaler.transform(X_test)

    return X_train_scaled, X_test_scaled, y_train, y_test

def getModelsDict():
    #lgbm classifier
    clf_lgbm = lgb.LGBMClassifier()

    # randon forest classifier
    clf_rf = RandomForestClassifier(max_depth=2)

    #knn classifier
    clf_knn = KNeighborsClassifier()

    #artificial neural net classifier

    #hyper parameters
    hidden_units=100
    learning_rate=0.01
    hidden_layer_act='tanh'
    output_layer_act='sigmoid'

    clf_ann = Sequential()
    clf_ann.add(Dense(hidden_units, input_dim=8, activation=hidden_layer_act))
    clf_ann.add(Dense(hidden_units, activation=hidden_layer_act))
    clf_ann.add(Dense(1, activation=output_layer_act))

    sgd=optimizers.SGD(lr=learning_rate)
    clf_ann.compile(loss='binary_crossentropy',optimizer=sgd, metrics=['acc'])
    
    model_dict = {'LGBM_Clf': clf_lgbm, 'KNN_Clf': clf_knn, 'ANN_Clf': clf_ann, 'RF_Clf': clf_rf}
    
    return model_dict


# In[4]:


def main():
    data = pd.read_csv('ClassificationDataset.csv')

    X_train, X_test, y_train, y_test = train_test_split(data)
    y_train = y_train['success'].to_numpy().tolist()
    y_test = y_test['success'].to_numpy().tolist()
    model_dict = getModelsDict()

    for model in model_dict:
        print(model)
        if model == 'ANN_Clf':
            model_dict[model].fit(X_train, y_train, epochs=100, batch_size=64,  verbose=2)
            model_dict[model].save('ANN_Clf')
            y_pred = model_dict[model].predict(X_test)
            y_pred = (y_pred > 0.5).astype(int).flatten()
        else: 
            model_dict[model].fit(X_train, y_train)
            pickle.dump(model_dict[model], open(model, 'wb'))
            y_pred = model_dict[model].predict(X_test)
        
        precision = precision_score(y_test, y_pred, average='binary')
        print("Precision: ", precision)
        
        recall = recall_score(y_test, y_pred, average='binary')
        print("Recall: ", recall)
        
        accuracy = accuracy_score(y_test, y_pred)
        print("Accuracy: ", accuracy)
        
        print("Confusion Matrix ")
        print(confusion_matrix(y_test, y_pred))
        print('__________________________________________________')


# In[5]:


if __name__=='__main__':
    main()


# In[ ]:







import numpy as np
import pickle
import pandas as pd
import optuna
from codecs import open

class Optimizer:
    """class for handling multi-objective optimization"""

    def __init__(self):
        optuna.logging.disable_default_handler()
        self.rgr = pickle.load(open('models/best_rgr.sav', 'rb'))
        self.rgr_scaler = pickle.load(open('models/scaler_rgr.pkl', 'rb'))
        self.clf = pickle.load(open('models/best_clf', 'rb'))
        self.clf_scaler = pickle.load(open('models/scaler_clf.pkl', 'rb'))
        self.test_case_type = None
        self.trials = 100

    def setTrailNumber(self, trials):
        self.trials = trials

    def setTestCaseType(self, tc_type):
        self.test_case_type = tc_type

    def getSolvabiltyStatus(self, array):
        return self.clf.predict(self.clf_scaler.transform(array))

    def getSolverPerformance(self, array):
        return self.rgr.predict(self.rgr_scaler.transform(array))

    def predictions(self, test_case_type, max_grid_points, newton_critical_tolerance, newton_armijo_probes,
                    newton_max_iterations,
                    newton_tolerance, add_factor, remove_factor, use_collocation_scaling):
        array = np.array([test_case_type, max_grid_points, newton_critical_tolerance,
                          newton_armijo_probes, newton_max_iterations, newton_tolerance,
                          add_factor, remove_factor, use_collocation_scaling])
        array = array.reshape(1, -1)

        if self.getSolvabiltyStatus(array) == 0:
            return np.array([[float('inf'), float('inf'), float('inf')]])
        else:
            return self.getSolverPerformance(array)

    def objective(self, trial):
        params = {'test_case_type': self.test_case_type,

                  'max_grid_points': trial.suggest_int('max_grid_points', 100, 10000),
                  'newton_critical_tolerance': trial.suggest_float('newton_critical_tolerance',
                                                                   np.log(1.0027016845009548e-12),
                                                                   np.log(9.93227151144808e+307)),
                  'newton_armijo_probes': trial.suggest_int('newton_armijo_probes', 1, 10),
                  'newton_max_iterations': trial.suggest_int('newton_max_iterations', 1, 100),
                  'newton_tolerance': trial.suggest_float('newton_tolerance',
                                                          np.log(1.0000700155834424e-12),
                                                          np.log(0.0099994154229888)),
                  'add_factor': trial.suggest_float('add_factor', 1, 1000),
                  'remove_factor': trial.suggest_float('remove_factor', 1.5422041344544035e-06,
                                                       1.9999960347604355),
                  'use_collocation_scaling': trial.suggest_categorical('use_collocation_scaling', [0, 1])}

        prediction = self.predictions(**params)

        return prediction[0][0], prediction[0][1], prediction[0][2]

    def optimizenODEevals(self, trial):
        x, y, z = self.objective(trial)
        return x

    def optimizenGridPoints(self, trial):
        x, y, z = self.objective(trial)
        return y

    def optimizemaxResiduum(self, trial):
        x, y, z = self.objective(trial)
        return z

    def optimize_ODEevals_gridPoints(self, trial):
        x, y, z = self.objective(trial)
        return x, y

    def optimize_gridPoints_maxResiduum(self, trial):
        x, y, z = self.objective(trial)
        return y, z

    def optimize_maxResiduum_ODEevals(self, trial):
        x, y, z = self.objective(trial)
        return z, x

    def singleCriterion(self, objective):
        study = optuna.create_study(directions=['minimize'])
        if objective == 'nODEevals':
            study.optimize(self.optimizenODEevals, n_trials=self.trials)
        elif objective == 'nGridPoints':
            study.optimize(self.optimizenGridPoints, n_trials=self.trials)
        else:
            study.optimize(self.optimizemaxResiduum, n_trials=self.trials)

        return study.best_trial

    def twoCriteria(self, objective):
        study = optuna.create_study(directions=['minimize', 'minimize'])
        if objective == 'EvalGp':
            study.optimize(self.optimize_ODEevals_gridPoints, n_trials=self.trials)
        elif objective == 'GpRes':
            study.optimize(self.optimize_gridPoints_maxResiduum, n_trials=self.trials)
        else:
            study.optimize(self.optimize_maxResiduum_ODEevals, n_trials=self.trials)

        return study.best_trials

    def threeCriteria(self):
        study = optuna.create_study(directions=['minimize', 'minimize', 'minimize'])
        study.optimize(self.objective, n_trials=self.trials)
        return study.best_trials


import plotly.express as px

class Visualizer():
    def __init__(self):
        """Init function"""
        
    def visualize2d(self, dataframe):
        fig = px.scatter(dataframe, x=dataframe.columns[-1], y=dataframe.columns[-2],
                        custom_data=[dataframe.columns[0], dataframe.columns[1], dataframe.columns[2],
                                    dataframe.columns[3], dataframe.columns[4], dataframe.columns[5],
                                    dataframe.columns[6], dataframe.columns[7], dataframe.columns[8]],
                        width=500, height=400)
        fig.update_traces(
            hovertemplate="<br>".join([
            "test_case_type: %{customdata[0]}",
            "max_grid_points: %{customdata[1]}",
            "newton_critical_tolerance: %{customdata[2]}",
            "newton_armijo_probes: %{customdata[3]}",
            "newton_max_iterations: %{customdata[4]}",
            "newton_tolerance: %{customdata[5]}",
            "add_factor: %{customdata[6]}",
            "remove_factor: %{customdata[7]}",
            "use_collocation_scaling: %{customdata[8]}",
            ])
        )
        
        fig.update_layout(
            margin=dict(l=20, r=20, t=20, b=20),
            paper_bgcolor="LightSteelBlue",
        )
        
        return fig.show()
        
    def visualize3d(self, dataframe):
        fig = px.scatter_3d(dataframe, x=dataframe.columns[-1], y=dataframe.columns[-2], z=dataframe.columns[-3],
                          custom_data=[dataframe.columns[0], dataframe.columns[1], dataframe.columns[2],
                                    dataframe.columns[3], dataframe.columns[4], dataframe.columns[5],
                                    dataframe.columns[6], dataframe.columns[7], dataframe.columns[8]],
                        width=500, height=400)
        
        fig.update_traces(
            hovertemplate="<br>".join([
            "test_case_type: %{customdata[0]}",
            "max_grid_points: %{customdata[1]}",
            "newton_critical_tolerance: %{customdata[2]}",
            "newton_armijo_probes: %{customdata[3]}",
            "newton_max_iterations: %{customdata[4]}",
            "newton_tolerance: %{customdata[5]}",
            "add_factor: %{customdata[6]}",
            "remove_factor: %{customdata[7]}",
            "use_collocation_scaling: %{customdata[8]}",
            ])
        )
        
        fig.update_layout(
            margin=dict(l=20, r=20, t=20, b=20),
            paper_bgcolor="LightSteelBlue",
        )
        
        return fig.show()


#!/usr/bin/env python
# coding: utf-8

# In[1]:


import pandas as pd
import numpy as np
from sklearn.model_selection import train_test_split
from sklearn.preprocessing import StandardScaler
import lightgbm as lgb
from sklearn.ensemble import RandomForestRegressor
from sklearn.neighbors import KNeighborsRegressor
import sklearn
from keras.models import Sequential
from keras.layers import Dense
from keras import optimizers
import keras
from sklearn.metrics import mean_squared_error, mean_absolute_percentage_error, r2_score
import pickle
from sklearn.multioutput import MultiOutputRegressor
from sklearn.preprocessing import QuantileTransformer
from sklearn.ensemble import RandomForestRegressor
from sklearn.compose import TransformedTargetRegressor


# In[2]:


def train_test_split(data):
    Input = pd.DataFrame()
    Output = pd.DataFrame()

    Input['test_case_type'] = data['test_case_type']
    Input['max_grid_points'] = data['max_grid_points']
    Input['newton_critical_tolerance'] = np.log(data['newton_critical_tolerance'])
    Input['newton_armijo_probes'] = data['newton_armijo_probes']
    Input['newton_max_iterations'] = data['newton_max_iterations']
    Input['newton_tolerance'] = np.log(data['newton_tolerance'])
    Input['add_factor'] = data['add_factor']
    Input['remove_factor'] = data['remove_factor']
    Input['use_collocation_scaling'] = data['use_collocation_scaling']

    Output['nODEevals'] = data['nODEevals']
    Output['nGridPoints'] = data['nGridPoints']
    Output['maxResiduum'] = data['maxResiduum']

    X_train, X_test, y_train, y_test = sklearn.model_selection.train_test_split(Input, Output, train_size = 0.8, 
                                                                                test_size = 0.2, random_state=101)
    
    #normalize the input features
    scaler = StandardScaler()
    X_train_scaled = scaler.fit_transform(X_train)
    X_test_scaled = scaler.transform(X_test)

    return X_train_scaled, X_test_scaled, y_train, y_test

def getModelsDict():
    #lgbm regressor
    
    # hyper parameters after tuning
    new_params = {
    'n_estimators':874, 
    'learning_rate':0.0954093125341308, 
    'min_child_samples':15, 
    'num_leaves':50
    }
    
    rgr_lgbm = MultiOutputRegressor(lgb.LGBMRegressor(**new_params), n_jobs=-1)

    # randon forest regressor
    rgr_rf = RandomForestRegressor(max_depth=2, random_state=0)

    #knn regressor
    rgr_knn = MultiOutputRegressor(KNeighborsRegressor(), n_jobs=-1)

    #artificial neural net regressor

    #hyper parameters
    hidden_units=250
    learning_rate=0.01
    hidden_layer_act='relu'
    output_layer_act='linear'

    rgr_ann = Sequential()

    rgr_ann.add(Dense(hidden_units, input_dim=9, activation=hidden_layer_act))
    rgr_ann.add(Dense(hidden_units, activation=hidden_layer_act))
    rgr_ann.add(Dense(hidden_units, activation=hidden_layer_act))
    rgr_ann.add(Dense(hidden_units, activation=hidden_layer_act))
    rgr_ann.add(Dense(hidden_units, activation=hidden_layer_act))
    rgr_ann.add(Dense(hidden_units, activation=hidden_layer_act))
    rgr_ann.add(Dense(3, activation=output_layer_act))

    sgd = optimizers.SGD(lr=learning_rate)
    rgr_ann.compile(loss='mean_absolute_error',optimizer=sgd, metrics=['mean_absolute_error'])
    
    model_dict = {'LGBM_Rgr': rgr_lgbm, 'KNN_Rgr': rgr_knn, 'ANN_Rgr': rgr_ann, 'RF_Rgr': rgr_rf}
    
    return model_dict


# In[3]:


def main():
    data = pd.read_csv('RegressionDataset.csv')

    X_train, X_test, y_train, y_test = train_test_split(data)
    model_dict = getModelsDict()
    
    #tranform the outputs to have normal distribution
    transformer = QuantileTransformer(output_distribution='normal')

    for model in model_dict:
        print(model)
        if model == 'ANN_Rgr':
            #explicitely transform the output distribution for neural n
            quantile = QuantileTransformer(output_distribution='normal')
            y_train_normal = quantile.fit_transform(y_train)
            model_dict[model].fit(X_train, y_train_normal, epochs=10, batch_size=32,  verbose=2)

            model_dict[model].save('ANN_Rgr')
            y_pred = model_dict[model].predict(X_test)
            # inverse tranformation to the original distribution
            y_pred = quantile.inverse_transform(y_pred)
        else:
            regr = TransformedTargetRegressor(regressor=model_dict[model],
                                  transformer=transformer)
            regr.fit(X_train, y_train)
            pickle.dump(regr, open(model, 'wb'))
            
            y_pred = regr.predict(X_test)
        
        print("Coefficient of determination ", r2_score(y_test, y_pred))
        print("Root Mean Squared Error ", mean_squared_error(y_test, y_pred, squared=False))
        print('Mean Absolute Percentage Error ', mean_absolute_percentage_error(y_test, y_pred))
        
        print('__________________________________________________')


# In[ ]:


if __name__=='__main__':
    main()


# In[ ]:




